import { Body, Controller, Get, HttpCode, NotFoundException, Param, ParseIntPipe, Patch, Query, Req, UseGuards } from '@nestjs/common'
import { PlayersService } from './players.service'
import { ApiBadRequestResponse, ApiBearerAuth, ApiNoContentResponse, ApiNotFoundResponse, ApiOkResponse, ApiUnauthorizedResponse } from '@nestjs/swagger'
import { PlayerEntity } from './entities/player.entity'
import { AuthenticatedRequest, JwtAuthGuard } from 'src/auth/jwt-auth.guard'
import { UpdatePasswordDto } from './dto/player_password.dto'
import { QueryPaginationDto } from 'src/common/dto/pagination-query.dto'
import { ApiOkResponsePaginated, paginateOutput } from 'src/common/utils/pagination.utils'
import { PaginateOutputDto } from 'src/common/dto/pagination-output.dto'

@Controller('players')
export class PlayersController {
  constructor(private readonly playersService: PlayersService) {}

  @Get()
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponsePaginated(PlayerEntity)
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  async findAll(@Query() queryPagination?: QueryPaginationDto): Promise<PaginateOutputDto<PlayerEntity>> {
    const [players, total] = await this.playersService.findAll(queryPagination)
    return paginateOutput<PlayerEntity>(players.map(player => new PlayerEntity(player)), total, queryPagination)
  }

  @Get(':id')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponse({ type: PlayerEntity })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiNotFoundResponse({ description: "Joueur⋅se non trouvé⋅e" })
  async findOne(@Param('id', ParseIntPipe) id: number) {
    const player = await this.playersService.findOne(id)
    if (!player)
      throw new NotFoundException(`Læ joueur⋅se avec l'identifiant ${id} n'existe pas`)
    return new PlayerEntity(player)
  }

  @Patch('/update-password')
  @HttpCode(204)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiNoContentResponse({description: "Le mot de passe a bien été modifié."})
  @ApiBadRequestResponse({description: "Erreur dans la saisie du nouveau mot de passe."})
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  async updatePassword(@Req() request: AuthenticatedRequest, @Body() body: UpdatePasswordDto) {
    await this.playersService.updatePassword(request.user, body)
  }
}
